/*
 * rsw.c - client program for remote switch service.
 * Written by Ron Beam, 1996.
 */

#include <stdio.h>
#include <rpc/rpc.h>  /* Standard RPC include file */
#include "switch.h"   /* This file is generated by rpcgen */

void usage(char *name)
{
	printf("Usage:\n");
	printf("\t%s [-v] [+abc] [-def] host\n",name);
	printf("\tWhere v (verbose) lists the state of all switches,\n");
	printf("\ta,b,c are switch numbers (0-7) to activate,\n");
	printf("\td,e,f are switch numbers (0-7) to deactivate,\n");
	printf("\tand host is the hostname of the switched system\n");
	exit(1);
}

int main(int argc, char *argv[])
{
	CLIENT		*cl;		/* RPC handle */
	char		*server = NULL;
	long		on_mask = 0L;
	long		off_mask = 0L;
	long		new_mask;
	long		*cur_mask;
	long		*lresult;
	int		verbose = 0;
	char		*name = argv[0];

	register char   *p;
	register char   sw;
	register int    i,j;

	if (argc < 2)
		usage(name);

	for (i=1;i<argc;i++)
	{
		p = argv[i];
		switch (*p)
		{
			case '+':
				if (!*(p+1))
				{
					on_mask = 0xFF;
					break;
				}
				for (j=1;*(p+j);j++)
				{
					sw = *(p+j);
					if (sw >= '0' && sw <= '7')
					{
						on_mask |= (1 << (sw-'0'));
					}
					else
						usage(name);
				}
				break;
			case '-':
				if (!*(p+1))
				{
					off_mask = 0xFF;
					break;
				}
				for (j=1;*(p+j);j++)
				{
					sw = *(p+j);
					if (sw >= '0' && sw <= '7')
					{
						off_mask |= (1 << (sw-'0'));
					}
					else if (sw == 'v' || sw == 'V')
						verbose = 1;
					else
						usage(name);
				}
				break;
			default:
				server = p;
				break;
		}
	}

	if (server == NULL)
		usage(name);

	/*
	 * Create the client "handle."
	 */

	if ((cl = clnt_create(server, SWITCH_PROG, SWITCH_VERS, "udp")) == NULL)
	{
		/*
		 * Could not establish connection with server.
		 */

		clnt_pcreateerror(server);
		exit(2);
	}

	/* Get current switch mask */
	if ((cur_mask = sw_stat_1(NULL, cl)) == NULL)
	{
		clnt_perror(cl, server);
		exit(2);
	}

	/* Compute switches to set */
	new_mask = (*cur_mask | on_mask) & ~off_mask;

	/* Set all desired switches */
	if (*cur_mask != new_mask)
	{
		if ((lresult = sw_set_1(&new_mask, cl)) == NULL)
		{
			clnt_perror(cl, server);
			exit(2);
		}
		new_mask = *lresult;
	}

	if (verbose)
	{
		printf("Switch\tState\n");
		for (i=0; i<8; i++)
		{
			printf("%d\t",i);
			if (new_mask & (1 << i))
				printf("ON\n");
			else
				printf("OFF\n");
		}
	}

	clnt_destroy(cl);		/* done with the handle */
	return 0;
}

